<?php

namespace Drupal\external_entities\Plugin\ExternalEntities\DataAggregator;

use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\external_entities\DataAggregator\DataAggregatorBase;
use Drupal\external_entities\DataAggregator\DataAggregatorInterface;
use Drupal\external_entities\Entity\ExternalEntityInterface;
use Drupal\external_entities\Form\XnttSubformState;

/**
 * External entities data aggregator that only uses a single storage client.
 *
 * @DataAggregator(
 *   id = "single",
 *   label = @Translation("Single storage client"),
 *   description = @Translation("Only use one storage client source.")
 * )
 */
class SingleStorageClient extends DataAggregatorBase implements DataAggregatorInterface {

  use MessengerTrait;

  /**
   * {@inheritdoc}
   */
  public function loadMultiple(array $ids = NULL) :array {
    if (empty($this->externalEntityType)) {
      // Warn for invalid initialization.
      $this->logger->warning(
        'SingleStorageClient::loadMultiple() called without proper initialization of the external entity type member.'
      );
      return [];
    }
    return $this->getStorageClient(0)->loadMultiple($ids);
  }

  /**
   * {@inheritdoc}
   */
  public function save(ExternalEntityInterface $entity) :int {
    if (empty($this->externalEntityType)) {
      // Warn for invalid initialization.
      $this->logger->warning(
        'SingleStorageClient::save() called without proper initialization of the external entity type member.'
      );
      throw new EntityStorageException(
        'SingleStorageClient::save() called without proper initialization of the external entity type member.'
      );
    }
    return $this->getStorageClient(0)->save($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function delete(ExternalEntityInterface $entity) {
    if (empty($this->externalEntityType)) {
      // Warn for invalid initialization.
      $this->logger->warning(
        'SingleStorageClient::delete() called without proper initialization of the external entity type member.'
      );
      return;
    }
    $this->getStorageClient(0)->delete($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function query(
    array $parameters = [],
    array $sorts = [],
    ?int $start = NULL,
    ?int $length = NULL
  ) :array {
    if (empty($this->externalEntityType)) {
      // Warn for invalid initialization.
      $this->logger->warning(
        'SingleStorageClient::query() called without proper initialization of the external entity type member.'
      );
      return [];
    }
    return $this->getStorageClient(0)->query($parameters, $sorts, $start, $length);
  }

  /**
   * {@inheritdoc}
   */
  public function countQuery(array $parameters = []) :int {
    if (empty($this->externalEntityType)) {
      // Warn for invalid initialization.
      $this->logger->warning(
        'SingleStorageClient::countQuery() called without proper initialization of the external entity type member.'
      );
      return [];
    }
    return $this->getStorageClient(0)->countQuery($parameters);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $this->buildStorageClientSelectForm($form, $form_state);
    $this->buildStorageClientConfigForm($form, $form_state);
    return $form;
  }

  /**
   * Build a form element for selecting a storage client.
   *
   * @param array &$form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  protected function buildStorageClientSelectForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $storage_clients = $this->storageClientManager->getDefinitions();
    if (empty($storage_clients)) {
      return;
    }

    if (!empty($this->externalEntityType)) {
      // Check if external entity type has locks.
      $lock_status = \Drupal::state()->get('external_entities.type.locked');
      if (!empty($lock_status[$this->externalEntityType->id()]['lock_data_aggregator']['lock_storage_clients'][0]['allow_plugins'])) {
        $storage_clients = array_filter(
          $storage_clients,
          function ($key) use ($lock_status) {
            return !empty($lock_status[$this->externalEntityType->id()]['lock_data_aggregator']['lock_storage_clients'][0]['allow_plugins']);
          },
          ARRAY_FILTER_USE_KEY
        );
      }
    }

    $current_storage_client_id =
      $form_state->getValue(['storage_clients', 0, 'id'])
      ?? ($this->getConfiguration()['storage_clients'][0]['id'] ?? '')
      ?: DataAggregatorBase::DEFAULT_STORAGE_CLIENT;
    $storage_client_options = [];
    foreach ($storage_clients as $storage_client_id => $definition) {
      $config = $this->getStorageClientDefaultConfiguration();
      $storage_client = $this->storageClientManager->createInstance($storage_client_id, $config);
      $storage_client_options[$storage_client_id] = $storage_client->getLabel();
    }
    asort($storage_client_options, SORT_NATURAL | SORT_FLAG_CASE);

    $container_id = 'storage_clients';
    $form['storage_clients'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => $container_id,
      ],
    ];

    $form['storage_clients'][0]['id'] = [
      '#type' => 'select',
      '#title' => $this->t('Storage client'),
      '#description' => $this->t(
        'Choose a storage client to use, then configure it below.'
      ),
      '#default_value' => $current_storage_client_id,
      '#empty_value' => '',
      '#options' => $storage_client_options,
      '#wrapper_attributes' => ['class' => ['xntt-inline']],
      '#attributes' => [
        'class' => ['xntt-field'],
        'autocomplete' => 'off',
      ],
      '#label_attributes' => ['class' => ['xntt-label']],
      '#ajax' => [
        'callback' => [get_class($this), 'buildAjaxParentSubForm'],
        'wrapper' => $container_id,
        'method' => 'replace',
        'effect' => 'fade',
      ],
    ];
  }

  /**
   * Build a form element for configuring a field property mapping.
   *
   * @param array &$form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  protected function buildStorageClientConfigForm(
    array &$form,
    FormStateInterface $form_state
  ) {
    $config = $this->getConfiguration();
    $storage_client_id =
      $form_state->getValue(['storage_clients', 0, 'id'])
      ?? ($config['storage_clients'][0]['id'] ?? '')
      ?: DataAggregatorBase::DEFAULT_STORAGE_CLIENT;

    // Check if config correspond to selected storage client.
    $sc_config = $this->getStorageClientDefaultConfiguration();
    if ($storage_client_id == ($config['storage_clients'][0]['id'] ?? '#')) {
      $sc_config += $config['storage_clients'][0]['config'];
    }

    // Generate a new storage client instance.
    $storage_client = $this->storageClientManager->createInstance($storage_client_id, $sc_config);
    $storage_client_form_state = XnttSubformState::createForSubform(['storage_clients', 0, 'config'], $form, $form_state);
    $form['storage_clients'][0]['config'] = $storage_client->buildConfigurationForm(
      $form['storage_clients'][0]['config'],
      $storage_client_form_state
    );
  }

}
